@@ -1,10 +1,11 @@
 import os
-import pathlib
 import re
 import typing
+import pathlib
 import datetime
-import functools
 import warnings
+import functools
+import configparser
 
 from clldutils.inifile import INI
 from newick import Node
@@ -31,14 +32,13 @@ ISO_8601_INTERVAL = re.compile(
 @functools.total_ordering
 class Languoid(object):
     """
-    Info on languoids is encoded in the ini files and in the directory hierarchy.
+    Info on languoids is encoded in the INI files and in the directory hierarchy of
+    :attr:`pyglottolog.Glottolog.tree`.
     This class provides access to all of it.
 
     **Languoid formatting**:
 
-    :ivar _format_specs: A `dict` mapping custom format specifiers to conversion functions.
-
-    Usage:
+    :ivar _format_specs: A `dict` mapping custom format specifiers to conversion functions. Usage:
 
     .. code-block:: python
 
@@ -87,13 +87,16 @@ class Languoid(object):
         self._api = _api
 
     @classmethod
-    def from_dir(cls, directory: pathlib.Path, nodes=None, **kw):
+    def from_dir(cls, directory: pathlib.Path, nodes=None, _api=None, **kw):
         """
         Create a `Languoid` from a directory, named with the Glottocode and containing `md.ini`.
 
         This method is used by :class:`pyglottolog.Glottolog` to read `Languoid`s from the
         repository's `languoids/tree` directory.
         """
+        if _api and _api.cache and directory.name in _api.cache:
+            return _api.cache[directory.name]
+
         if nodes is None:
             nodes = {}
         cfg = INI.from_file(directory.joinpath(INFO_FILENAME), interpolation=None)
@@ -107,18 +110,18 @@ class Languoid(object):
                 break
 
             if id_ not in nodes:
-                l_ = Languoid.from_dir(parent, nodes=nodes, **kw)
+                l_ = Languoid.from_dir(parent, nodes=nodes, _api=_api, **kw)
                 nodes[id_] = (l_.name, l_.id, l_.level)
             lineage.append(nodes[id_])
 
-        res = cls(cfg, list(reversed(lineage)), directory=directory, **kw)
+        res = cls(cfg, list(reversed(lineage)), directory=directory, _api=_api, **kw)
         nodes[res.id] = (res.name, res.id, res.level)
         return res
 
     @classmethod
     def from_name_id_level(cls, tree, name, id, level, **kw):
         """
-        This method is used in `pyglottolog.lff` to instantiate `Languoid`s for new nodes
+        This method is used in `pyglottolog.lff` to instantiate `Languoid` s for new nodes
         encountered in "lff"-format trees.
         """
         cfg = INI(interpolation=None)
@@ -153,7 +156,7 @@ class Languoid(object):
         return object.__format__(self, format_spec)
 
     def __hash__(self):
-        return id(self)
+        return hash(self.id)
 
     def __eq__(self, other):
         return self.id == other.id
@@ -229,6 +232,7 @@ class Languoid(object):
     # -------------------------------------------------------------------------
     @property
     def glottocode(self):
+        """Alias for `id`"""
         return self._id
 
     @property
@@ -237,6 +241,14 @@ class Languoid(object):
 
     @property
     def category(self):
+        """
+        Languoid category.
+
+        - Category name from :class:`pyglottolog.config.LanguageType` for languoids of level \
+          "language",
+        - `"Family"` or `"Pseudo Family"` for families,
+        - `"Dialect"` for dialects.
+        """
         # Computing the category requires access to config data:
         if self._api:
             pseudo_families = {
@@ -252,7 +264,11 @@ class Languoid(object):
             return cat
 
     @property
-    def isolate(self):
+    def isolate(self) -> bool:
+        """
+        Flag signaling whether the languoid is an isolate, i.e. has level "language" and is not
+        member of a family.
+        """
         return getattr(self.level, 'id', self.level) == 'language' and not self.lineage
 
     def children_from_nodemap(self, nodes):
@@ -269,7 +285,16 @@ class Languoid(object):
             ((level is None) or n.level == level)]
 
     @property
-    def children(self):
+    def children(self) -> typing.List['Languoid']:
+        """
+        List of direct descendants of the languoid in the classification tree.
+
+        .. note::
+
+            Using this on many languoids can be slow, because the directory tree may be traversed
+            and INI files read multiple times. To circumvent this problem, you may use a read-only
+            :class:`pyglottolog.Glottolog` instance, by passing `cache=True` at initialization.
+        """
         return [Languoid.from_dir(d, _api=self._api) for d in self.dir.iterdir() if d.is_dir()]
 
     def ancestors_from_nodemap(self, nodes):
@@ -277,30 +302,69 @@ class Languoid(object):
         # been read from disc.
         return [nodes[lineage[1]] for lineage in self.lineage]
 
-    @property
-    def ancestors(self):
-        res = []
+    def iter_ancestors(self):
         for parent in self.dir.parents:
             id_ = parent.name
             if Glottocode.pattern.match(id_):
-                res.append(Languoid.from_dir(parent, _api=self._api))
+                yield Languoid.from_dir(parent, _api=self._api)
             else:
                 # we ignore leading non-languoid-dir path components.
                 break
-        return list(reversed(res))
+
+    def iter_descendants(self):
+        for child in self.children:
+            yield child
+            yield from child.iter_descendants()
 
     @property
-    def parent(self):
-        ancestors = self.ancestors
-        return ancestors[-1] if ancestors else None
+    def ancestors(self) -> typing.List['Languoid']:
+        """
+        List of ancestors of the languoid in the classification tree, from root (i.e. top-level
+        family) to parent node.
+
+        .. note::
+
+            Using this on many languoids can be slow, because the directory tree may be traversed
+            and INI files read multiple times. To circumvent this problem, you may use a read-only
+            :class:`pyglottolog.Glottolog` instance, by passing `cache=True` at initialization.
+        """
+        return list(reversed(list(self.iter_ancestors())))
 
     @property
-    def family(self):
-        ancestors = self.ancestors
-        return ancestors[0] if ancestors else None
+    def parent(self) -> typing.Union['Languoid', None]:
+        """
+        Parent languoid or `None`.
+
+        .. note::
+
+            Using this on many languoids can be slow, because the directory tree may be traversed
+            and INI files read multiple times. To circumvent this problem, you may use a read-only
+            :class:`pyglottolog.Glottolog` instance, by passing `cache=True` at initialization.
+        """
+        try:
+            return next(self.iter_ancestors())
+        except StopIteration:
+            return
 
     @property
-    def names(self):
+    def family(self) -> typing.Union['Languoid', None]:
+        """
+        Top-level family the languoid belongs to or `None`.
+
+        .. note::
+
+            Using this on many languoids can be slow, because the directory tree may be traversed
+            and INI files read multiple times. To circumvent this problem, you may use a read-only
+            :class:`pyglottolog.Glottolog` instance, by passing `cache=True` at initialization.
+        """
+        return self.ancestors[0] if self.lineage else None
+
+    @property
+    def names(self) -> typing.Dict[str, list]:
+        """
+        A `dict` mapping alternative name providers to `list` s of alternative names for the
+        languoid by the given provider.
+        """
         if 'altnames' in self.cfg:
             return {k: self.cfg.getlist('altnames', k) for k in self.cfg['altnames']}
         return {}
@@ -318,13 +382,18 @@ class Languoid(object):
         return False
 
     @property
-    def identifier(self):
+    def identifier(self) -> typing.Union[dict, configparser.SectionProxy]:
         if 'identifier' in self.cfg:
             return self.cfg['identifier']
         return {}
 
     @property
-    def sources(self):
+    def sources(self) -> typing.List[Reference]:
+        """
+        List of Glottolog references linked to the languoid
+
+        :rtype: :class:`pyglottolog.references.Reference`
+        """
         if self.cfg.has_option('sources', 'glottolog'):
             return Reference.from_list(self.cfg.getlist('sources', 'glottolog'))
         return []
@@ -335,7 +404,12 @@ class Languoid(object):
         self.cfg.set('sources', 'glottolog', ['{0}'.format(ref) for ref in refs])
 
     @property
-    def endangerment(self):
+    def endangerment(self) -> typing.Union[None, Endangerment]:
+        """
+        Endangerment information about the languoid.
+
+        :rtype: :class:`Endangerment`
+        """
         if ('endangerment' in self.cfg) and self._api:
             kw = {k: v for k, v in self.cfg['endangerment'].items()}
             kw['status'] = self._api.aes_status.get(kw['status'])
@@ -352,7 +426,12 @@ class Languoid(object):
             return Endangerment(**kw)
 
     @property
-    def classification_comment(self):
+    def classification_comment(self) -> typing.Union[None, ClassificationComment]:
+        """
+        Classification information about the languoid.
+
+        :rtype: :class:`ClassificationComment`
+        """
         if 'classification' in self.cfg:
             cfg = self.cfg['classification']
             return ClassificationComment(
@@ -362,13 +441,21 @@ class Languoid(object):
                 subrefs=self.cfg.getlist('classification', 'subrefs'))
 
     @property
-    def ethnologue_comment(self):
+    def ethnologue_comment(self) -> typing.Union[None, EthnologueComment]:
+        """
+        Commentary about the classification of the languoid in Ethnologue.
+
+        :rtype: :class:`EthnologueComment`
+        """
         section = 'hh_ethnologue_comment'
         if section in self.cfg:
             return EthnologueComment(**self.cfg[section])
 
     @property
-    def macroareas(self):
+    def macroareas(self) -> typing.List[config.Macroarea]:
+        """
+        :rtype: `list` of :class:`config.Macroarea`
+        """
         if self._api:
             return [
                 self._api.macroareas.get(n)
@@ -428,7 +515,10 @@ class Languoid(object):
         self._set('timespan', '{}-01-01/{}-01-01'.format(*map(fmt, value)))
 
     @property
-    def links(self):
+    def links(self) -> typing.List[Link]:
+        """
+        Links to web resources related to the languoid
+        """
         return [Link.from_string(s) for s in self.cfg.getlist(self.section_core, 'links')]
 
     @links.setter
@@ -444,7 +534,10 @@ class Languoid(object):
         return False
 
     @property
-    def countries(self):
+    def countries(self) -> typing.List[Country]:
+        """
+        Countries a language is spoken in.
+        """
         return [Country.from_text(n)
                 for n in self.cfg.getlist(self.section_core, 'countries')]
 
@@ -456,6 +549,9 @@ class Languoid(object):
 
     @property
     def name(self):
+        """
+        The Glottolog mame of the languoid
+        """
         return self._get('name')
 
     @name.setter
@@ -463,7 +559,10 @@ class Languoid(object):
         self._set('name', value)
 
     @property
-    def latitude(self):
+    def latitude(self) -> typing.Union[None, float]:
+        """
+        The geographic latitude of the point chosen as representative coordinate of the languoid
+        """
         return self._get('latitude', float)
 
     @latitude.setter
@@ -471,7 +570,10 @@ class Languoid(object):
         self._set('latitude', round(float(value), 5))
 
     @property
-    def longitude(self):
+    def longitude(self) -> typing.Union[None, float]:
+        """
+        The geographic longitude of the point chosen as representative coordinate of the languoid
+        """
         return self._get('longitude', float)
 
     @longitude.setter
@@ -513,6 +615,22 @@ class Languoid(object):
     def iso_code(self, value):
         self._set('iso639-3', value)
 
+    def closest_iso(self, api=None) -> typing.Union[str, None]:
+        """
+        ISO 639-3 code assigned to the languoid or one of its ancestors in the classification
+        (in case of dialects) or `None`.
+
+        .. seealso:: https://github.com/glottolog/glottolog-cldf/issues/13
+        """
+        api = api or self._api
+        assert api
+        if self.iso:
+            return self.iso
+        for _, gc, _ in reversed(self.lineage):
+            lg = api.languoid(gc)
+            if lg.iso:
+                return lg.iso
+
     @property
     def iso_retirement(self):
         if 'iso_retirement' in self.cfg:
